<?php
/*  Copyright (C) 2023      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       htdocs/banking4dolibarr/core/triggers/interface_90_modBanking4Dolibarr_Banking4Dolibarr.class.php
 *  \ingroup    banking4dolibarr
 *	\brief      File of class of triggers for banking4dolibarr module
 */


require_once DOL_DOCUMENT_ROOT.'/core/triggers/dolibarrtriggers.class.php';


/**
 *  Class of triggers for Banking4Dolibarr module
 */
class InterfaceBanking4Dolibarr extends DolibarrTriggers
{
	public $family = 'banking4dolibarr';
	public $description = "Triggers of this module catch triggers event for Banking4Dolibarr module.";
	public $version = self::VERSION_DOLIBARR;
	public $picto = 'technic';


	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "runTrigger" are triggered if file is inside directory htdocs/core/triggers or htdocs/module/code/triggers (and declared)
	 *
	 * @param string		$action		Event action code
	 * @param Object		$object     Object
	 * @param User		    $user       Object user
	 * @param Translate 	$langs      Object langs
	 * @param conf		    $conf       Object conf
	 * @return int         				<0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
	{
		if (empty($conf->banking4dolibarr->enabled)) return 0;     // Module not active, we do nothing

		switch ($action) {
			// Bank account line
			case 'BANKACCOUNTLINE_DELETE':
				$sql = "SELECT bbr.id_record";
				$sql .= " FROM " . MAIN_DB_PREFIX . "banking4dolibarr_bank_record_link AS bbrl";
				$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "banking4dolibarr_bank_record AS bbr ON bbr.rowid = bbrl.fk_bank_record";
				$sql .= " WHERE bbrl.fk_bank = " . ((int) $object->id);

				$resql = $this->db->query($sql);
				if (!$resql) {
					$this->errors[] = $this->db->lasterror();
					return -1;
				}

				$langs->load('banking4dolibarr@banking4dolibarr');
				$errors = array();
				while ($obj = $this->db->fetch_object($resql)) {
					$errors[] = $langs->trans('Banking4DolibarrErrorDeleteNotPossibleLineIsLinked', $obj->id_record);
				}
				$this->db->free($resql);

				if (!empty($errors)) {
					setEventMessages(null, $errors, 'errors');
					$this->errors = $errors;
					return -1;
				}

				dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
				break;
		}

		return 0;
	}
}